@@ -3,21 +3,23 @@
  * float.c
  *	  Functions for the built-in floating-point types.
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/float.c,v 1.82 2002/10/19 02:08:17 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/adt/float.c,v 1.115 2005/10/15 02:49:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 /*----------
  * OLD COMMENTS
  *		Basic float4 ops:
- *		 float4in, float4out, float4abs, float4um, float4up
+ *		 float4in, float4out, float4recv, float4send
+ *		 float4abs, float4um, float4up
  *		Basic float8 ops:
- *		 float8in, float8out, float8abs, float8um, float8up
+ *		 float8in, float8out, float8recv, float8send
+ *		 float8abs, float8um, float8up
  *		Arithmetic operators:
  *		 float4pl, float4mi, float4mul, float4div
  *		 float8pl, float8mi, float8mul, float8div
@@ -52,9 +54,8 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <float.h>				/* faked on sunos4 */
+#include <float.h>
 #include <math.h>
-
 #include <limits.h>
 /* for finite() on Solaris */
 #ifdef HAVE_IEEEFP_H
@@ -63,49 +64,16 @@
 
 #include "catalog/pg_type.h"
 #include "fmgr.h"
+#include "libpq/pqformat.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
 
 
-#if !(NeXT && NX_CURRENT_COMPILER_RELEASE > NX_COMPILER_RELEASE_3_2)
- /* NS3.3 has conflicting declarations of these in <math.h> */
-
-#ifndef atof
-extern double atof(const char *p);
-#endif
-
-#ifndef HAVE_CBRT
-#define cbrt my_cbrt
-static double cbrt(double x);
-
-#else
-#if !defined(nextstep)
-extern double cbrt(double x);
-#endif
-#endif   /* HAVE_CBRT */
-
-#ifndef HAVE_RINT
-#define rint my_rint
-static double rint(double x);
-
-#else
-extern double rint(double x);
-#endif   /* HAVE_RINT */
-#endif   /* NeXT check */
-
-
-static void CheckFloat4Val(double val);
-static void CheckFloat8Val(double val);
-
 #ifndef M_PI
 /* from my RH5.2 gcc math.h file - thomas 2000-04-03 */
 #define M_PI 3.14159265358979323846
 #endif
 
-#ifndef NAN
-#define NAN		(0.0/0.0)
-#endif
-
 #ifndef SHRT_MAX
 #define SHRT_MAX 32767
 #endif
@@ -113,8 +81,12 @@ static void CheckFloat8Val(double val);
 #define SHRT_MIN (-32768)
 #endif
 
-#define FORMAT			'g'		/* use "g" output format as standard
-								 * format */
+/* Recent HPUXen have isfinite() macro in place of more standard finite() */
+#if !defined(HAVE_FINITE) && defined(isfinite)
+#define finite(x) isfinite(x)
+#define HAVE_FINITE 1
+#endif
+
 /* not sure what the following should be, but better to make it over-sufficient */
 #define MAXFLOATWIDTH	64
 #define MAXDOUBLEWIDTH	128
@@ -128,51 +100,146 @@ static void CheckFloat8Val(double val);
 #define FLOAT8_MIN		 DBL_MIN
 
 
+/* Configurable GUC parameter */
+int			extra_float_digits = 0;		/* Added to DBL_DIG or FLT_DIG */
+
+
+static void CheckFloat4Val(double val);
+static void CheckFloat8Val(double val);
+static int	float4_cmp_internal(float4 a, float4 b);
+static int	float8_cmp_internal(float8 a, float8 b);
+
+#ifndef HAVE_CBRT
+static double cbrt(double x);
+#endif   /* HAVE_CBRT */
+
+
 /*
- * check to see if a float4 val is outside of
- * the FLOAT4_MIN, FLOAT4_MAX bounds.
+ * Routines to provide reasonably platform-independent handling of
+ * infinity and NaN.  We assume that isinf() and isnan() are available
+ * and work per spec.  (On some platforms, we have to supply our own;
+ * see src/port.)  However, generating an Infinity or NaN in the first
+ * place is less well standardized; pre-C99 systems tend not to have C99's
+ * INFINITY and NAN macros.  We centralize our workarounds for this here.
+ */
+
+double
+get_float8_infinity(void)
+{
+#ifdef INFINITY
+	/* C99 standard way */
+	return (double) INFINITY;
+#else
+
+	/*
+	 * On some platforms, HUGE_VAL is an infinity, elsewhere it's just the
+	 * largest normal double.  We assume forcing an overflow will get us a
+	 * true infinity.
+	 */
+	return (double) (HUGE_VAL * HUGE_VAL);
+#endif
+}
+
+float
+get_float4_infinity(void)
+{
+#ifdef INFINITY
+	/* C99 standard way */
+	return (float) INFINITY;
+#else
+
+	/*
+	 * On some platforms, HUGE_VAL is an infinity, elsewhere it's just the
+	 * largest normal double.  We assume forcing an overflow will get us a
+	 * true infinity.
+	 */
+	return (float) (HUGE_VAL * HUGE_VAL);
+#endif
+}
+
+double
+get_float8_nan(void)
+{
+#ifdef NAN
+	/* C99 standard way */
+	return (double) NAN;
+#else
+	/* Assume we can get a NAN via zero divide */
+	return (double) (0.0 / 0.0);
+#endif
+}
+
+float
+get_float4_nan(void)
+{
+#ifdef NAN
+	/* C99 standard way */
+	return (float) NAN;
+#else
+	/* Assume we can get a NAN via zero divide */
+	return (float) (0.0 / 0.0);
+#endif
+}
+
+
+/*
+ * Returns -1 if 'val' represents negative infinity, 1 if 'val'
+ * represents (positive) infinity, and 0 otherwise. On some platforms,
+ * this is equivalent to the isinf() macro, but not everywhere: C99
+ * does not specify that isinf() needs to distinguish between positive
+ * and negative infinity.
+ */
+int
+is_infinite(double val)
+{
+	int			inf = isinf(val);
+
+	if (inf == 0)
+		return 0;
+
+	if (val > 0)
+		return 1;
+
+	return -1;
+}
+
+
+/*
+ * check to see if a float4 val is outside of the FLOAT4_MIN,
+ * FLOAT4_MAX bounds.
  *
- * raise an elog warning if it is
-*/
+ * raise an ereport() error if it is
+ */
 static void
 CheckFloat4Val(double val)
 {
-	/*
-	 * defining unsafe floats's will make float4 and float8 ops faster at
-	 * the cost of safety, of course!
-	 */
-#ifdef UNSAFE_FLOATS
-	return;
-#else
 	if (fabs(val) > FLOAT4_MAX)
-		elog(ERROR, "Bad float4 input format -- overflow");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"real\" value out of range: overflow")));
 	if (val != 0.0 && fabs(val) < FLOAT4_MIN)
-		elog(ERROR, "Bad float4 input format -- underflow");
-	return;
-#endif   /* UNSAFE_FLOATS */
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("type \"real\" value out of range: underflow")));
 }
 
 /*
- * check to see if a float8 val is outside of
- * the FLOAT8_MIN, FLOAT8_MAX bounds.
+ * check to see if a float8 val is outside of the FLOAT8_MIN,
+ * FLOAT8_MAX bounds.
  *
- * raise an elog error if it is
+ * raise an ereport() error if it is
  */
 static void
 CheckFloat8Val(double val)
 {
-	/*
-	 * defining unsafe floats's will make float4 and float8 ops faster at
-	 * the cost of safety, of course!
-	 */
-#ifdef UNSAFE_FLOATS
-	return;
-#else
 	if (fabs(val) > FLOAT8_MAX)
-		elog(ERROR, "Bad float8 input format -- overflow");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+		  errmsg("type \"double precision\" value out of range: overflow")));
 	if (val != 0.0 && fabs(val) < FLOAT8_MIN)
-		elog(ERROR, "Bad float8 input format -- underflow");
-#endif   /* UNSAFE_FLOATS */
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+		 errmsg("type \"double precision\" value out of range: underflow")));
 }
 
 /*
@@ -186,34 +253,98 @@ Datum
 float4in(PG_FUNCTION_ARGS)
 {
 	char	   *num = PG_GETARG_CSTRING(0);
+	char	   *orig_num;
 	double		val;
 	char	   *endptr;
 
-	errno = 0;
-	val = strtod(num, &endptr);
-	if (*endptr != '\0')
-	{
-		/*
-		 * XXX we should accept "Infinity" and "-Infinity" too, but what
-		 * are the correct values to assign?  HUGE_VAL will provoke an
-		 * error from CheckFloat4Val.
-		 */
-		if (strcasecmp(num, "NaN") == 0)
-			val = NAN;
-		else
-			elog(ERROR, "Bad float4 input format '%s'", num);
-	}
-	else
-	{
-		if (errno == ERANGE)
-			elog(ERROR, "Input '%s' is out of range for float4", num);
-	}
+	/*
+	 * endptr points to the first character _after_ the sequence we recognized
+	 * as a valid floating point number. orig_num points to the original input
+	 * string.
+	 */
+	orig_num = num;
 
 	/*
-	 * if we get here, we have a legal double, still need to check to see
-	 * if it's a legal float
+	 * Check for an empty-string input to begin with, to avoid the vagaries of
+	 * strtod() on different platforms.
 	 */
-	CheckFloat4Val(val);
+	if (*num == '\0')
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+				 errmsg("invalid input syntax for type real: \"%s\"",
+						orig_num)));
+
+	/* skip leading whitespace */
+	while (*num != '\0' && isspace((unsigned char) *num))
+		num++;
+
+	errno = 0;
+	val = strtod(num, &endptr);
+
+	/* did we not see anything that looks like a double? */
+	if (endptr == num || errno != 0)
+	{
+		/*
+		 * C99 requires that strtod() accept NaN and [-]Infinity, but not all
+		 * platforms support that yet (and some accept them but set ERANGE
+		 * anyway...)  Therefore, we check for these inputs ourselves.
+		 */
+		if (pg_strncasecmp(num, "NaN", 3) == 0)
+		{
+			val = get_float4_nan();
+			endptr = num + 3;
+		}
+		else if (pg_strncasecmp(num, "Infinity", 8) == 0)
+		{
+			val = get_float4_infinity();
+			endptr = num + 8;
+		}
+		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
+		{
+			val = -get_float4_infinity();
+			endptr = num + 9;
+		}
+		else if (errno == ERANGE)
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+					 errmsg("\"%s\" is out of range for type real",
+							orig_num)));
+		else
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+					 errmsg("invalid input syntax for type real: \"%s\"",
+							orig_num)));
+	}
+#ifdef HAVE_BUGGY_SOLARIS_STRTOD
+	else
+	{
+		/*
+		 * Many versions of Solaris have a bug wherein strtod sets endptr to
+		 * point one byte beyond the end of the string when given "inf" or
+		 * "infinity".
+		 */
+		if (endptr != num && endptr[-1] == '\0')
+			endptr--;
+	}
+#endif   /* HAVE_BUGGY_SOLARIS_STRTOD */
+
+	/* skip trailing whitespace */
+	while (*endptr != '\0' && isspace((unsigned char) *endptr))
+		endptr++;
+
+	/* if there is any junk left at the end of the string, bail out */
+	if (*endptr != '\0')
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+				 errmsg("invalid input syntax for type real: \"%s\"",
+						orig_num)));
+
+	/*
+	 * if we get here, we have a legal double, still need to check to see if
+	 * it's a legal float4
+	 */
+	if (!isinf(val))
+		CheckFloat4Val(val);
 
 	PG_RETURN_FLOAT4((float4) val);
 }
@@ -227,20 +358,57 @@ float4out(PG_FUNCTION_ARGS)
 {
 	float4		num = PG_GETARG_FLOAT4(0);
 	char	   *ascii = (char *) palloc(MAXFLOATWIDTH + 1);
-	int			infflag;
 
 	if (isnan(num))
 		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
-	infflag = isinf(num);
-	if (infflag > 0)
-		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
-	if (infflag < 0)
-		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
 
-	sprintf(ascii, "%.*g", FLT_DIG, num);
+	switch (is_infinite(num))
+	{
+		case 1:
+			strcpy(ascii, "Infinity");
+			break;
+		case -1:
+			strcpy(ascii, "-Infinity");
+			break;
+		default:
+			{
+				int			ndig = FLT_DIG + extra_float_digits;
+
+				if (ndig < 1)
+					ndig = 1;
+
+				sprintf(ascii, "%.*g", ndig, num);
+			}
+	}
+
 	PG_RETURN_CSTRING(ascii);
 }
 
+/*
+ *		float4recv			- converts external binary format to float4
+ */
+Datum
+float4recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_FLOAT4(pq_getmsgfloat4(buf));
+}
+
+/*
+ *		float4send			- converts float4 to binary format
+ */
+Datum
+float4send(PG_FUNCTION_ARGS)
+{
+	float4		num = PG_GETARG_FLOAT4(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendfloat4(&buf, num);
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
 /*
  *		float8in		- converts "num" to float8
  *						  restricted syntax:
@@ -252,34 +420,98 @@ Datum
 float8in(PG_FUNCTION_ARGS)
 {
 	char	   *num = PG_GETARG_CSTRING(0);
+	char	   *orig_num;
 	double		val;
 	char	   *endptr;
 
+	/*
+	 * endptr points to the first character _after_ the sequence we recognized
+	 * as a valid floating point number. orig_num points to the original input
+	 * string.
+	 */
+	orig_num = num;
+
+	/*
+	 * Check for an empty-string input to begin with, to avoid the vagaries of
+	 * strtod() on different platforms.
+	 */
+	if (*num == '\0')
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+			 errmsg("invalid input syntax for type double precision: \"%s\"",
+					orig_num)));
+
+	/* skip leading whitespace */
+	while (*num != '\0' && isspace((unsigned char) *num))
+		num++;
+
 	errno = 0;
 	val = strtod(num, &endptr);
-	if (*endptr != '\0')
+
+	/* did we not see anything that looks like a double? */
+	if (endptr == num || errno != 0)
 	{
-		if (strcasecmp(num, "NaN") == 0)
-			val = NAN;
-		else if (strcasecmp(num, "Infinity") == 0)
-			val = HUGE_VAL;
-		else if (strcasecmp(num, "-Infinity") == 0)
-			val = -HUGE_VAL;
+		/*
+		 * C99 requires that strtod() accept NaN and [-]Infinity, but not all
+		 * platforms support that yet (and some accept them but set ERANGE
+		 * anyway...)  Therefore, we check for these inputs ourselves.
+		 */
+		if (pg_strncasecmp(num, "NaN", 3) == 0)
+		{
+			val = get_float8_nan();
+			endptr = num + 3;
+		}
+		else if (pg_strncasecmp(num, "Infinity", 8) == 0)
+		{
+			val = get_float8_infinity();
+			endptr = num + 8;
+		}
+		else if (pg_strncasecmp(num, "-Infinity", 9) == 0)
+		{
+			val = -get_float8_infinity();
+			endptr = num + 9;
+		}
+		else if (errno == ERANGE)
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				   errmsg("\"%s\" is out of range for type double precision",
+						  orig_num)));
 		else
-			elog(ERROR, "Bad float8 input format '%s'", num);
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+			 errmsg("invalid input syntax for type double precision: \"%s\"",
+					orig_num)));
 	}
+#ifdef HAVE_BUGGY_SOLARIS_STRTOD
 	else
 	{
-		if (errno == ERANGE)
-			elog(ERROR, "Input '%s' is out of range for float8", num);
+		/*
+		 * Many versions of Solaris have a bug wherein strtod sets endptr to
+		 * point one byte beyond the end of the string when given "inf" or
+		 * "infinity".
+		 */
+		if (endptr != num && endptr[-1] == '\0')
+			endptr--;
 	}
+#endif   /* HAVE_BUGGY_SOLARIS_STRTOD */
 
-	CheckFloat8Val(val);
+	/* skip trailing whitespace */
+	while (*endptr != '\0' && isspace((unsigned char) *endptr))
+		endptr++;
+
+	/* if there is any junk left at the end of the string, bail out */
+	if (*endptr != '\0')
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+			 errmsg("invalid input syntax for type double precision: \"%s\"",
+					orig_num)));
+
+	if (!isinf(val))
+		CheckFloat8Val(val);
 
 	PG_RETURN_FLOAT8(val);
 }
 
-
 /*
  *		float8out		- converts float8 number to a string
  *						  using a standard output format
@@ -289,20 +521,58 @@ float8out(PG_FUNCTION_ARGS)
 {
 	float8		num = PG_GETARG_FLOAT8(0);
 	char	   *ascii = (char *) palloc(MAXDOUBLEWIDTH + 1);
-	int			infflag;
 
 	if (isnan(num))
 		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
-	infflag = isinf(num);
-	if (infflag > 0)
-		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
-	if (infflag < 0)
-		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
 
-	sprintf(ascii, "%.*g", DBL_DIG, num);
+	switch (is_infinite(num))
+	{
+		case 1:
+			strcpy(ascii, "Infinity");
+			break;
+		case -1:
+			strcpy(ascii, "-Infinity");
+			break;
+		default:
+			{
+				int			ndig = DBL_DIG + extra_float_digits;
+
+				if (ndig < 1)
+					ndig = 1;
+
+				sprintf(ascii, "%.*g", ndig, num);
+			}
+	}
+
 	PG_RETURN_CSTRING(ascii);
 }
 
+/*
+ *		float8recv			- converts external binary format to float8
+ */
+Datum
+float8recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_FLOAT8(pq_getmsgfloat8(buf));
+}
+
+/*
+ *		float8send			- converts float8 to binary format
+ */
+Datum
+float8send(PG_FUNCTION_ARGS)
+{
+	float8		num = PG_GETARG_FLOAT8(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendfloat8(&buf, num);
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
+
 /* ========== PUBLIC ROUTINES ========== */
 
 
@@ -349,7 +619,10 @@ float4larger(PG_FUNCTION_ARGS)
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 	float4		result;
 
-	result = ((arg1 > arg2) ? arg1 : arg2);
+	if (float4_cmp_internal(arg1, arg2) > 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT4(result);
 }
 
@@ -360,7 +633,10 @@ float4smaller(PG_FUNCTION_ARGS)
 	float4		arg2 = PG_GETARG_FLOAT4(1);
 	float4		result;
 
-	result = ((arg1 < arg2) ? arg1 : arg2);
+	if (float4_cmp_internal(arg1, arg2) < 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT4(result);
 }
 
@@ -416,8 +692,10 @@ float8larger(PG_FUNCTION_ARGS)
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 	float8		result;
 
-	result = ((arg1 > arg2) ? arg1 : arg2);
-
+	if (float8_cmp_internal(arg1, arg2) > 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT8(result);
 }
 
@@ -428,8 +706,10 @@ float8smaller(PG_FUNCTION_ARGS)
 	float8		arg2 = PG_GETARG_FLOAT8(1);
 	float8		result;
 
-	result = ((arg1 < arg2) ? arg1 : arg2);
-
+	if (float8_cmp_internal(arg1, arg2) < 0)
+		result = arg1;
+	else
+		result = arg2;
 	PG_RETURN_FLOAT8(result);
 }
 
@@ -490,7 +770,9 @@ float4div(PG_FUNCTION_ARGS)
 	double		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float4div: divide by zero error");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	/* Do division in float8, then check for overflow */
 	result = (float8) arg1 / (float8) arg2;
@@ -552,7 +834,9 @@ float8div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float8div: divide by zero error");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 
@@ -574,9 +858,9 @@ static int
 float4_cmp_internal(float4 a, float4 b)
 {
 	/*
-	 * We consider all NANs to be equal and larger than any non-NAN. This
-	 * is somewhat arbitrary; the important thing is to have a consistent
-	 * sort order.
+	 * We consider all NANs to be equal and larger than any non-NAN. This is
+	 * somewhat arbitrary; the important thing is to have a consistent sort
+	 * order.
 	 */
 	if (isnan(a))
 	{
@@ -670,9 +954,9 @@ static int
 float8_cmp_internal(float8 a, float8 b)
 {
 	/*
-	 * We consider all NANs to be equal and larger than any non-NAN. This
-	 * is somewhat arbitrary; the important thing is to have a consistent
-	 * sort order.
+	 * We consider all NANs to be equal and larger than any non-NAN. This is
+	 * somewhat arbitrary; the important thing is to have a consistent sort
+	 * order.
 	 */
 	if (isnan(a))
 	{
@@ -759,6 +1043,26 @@ btfloat8cmp(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32(float8_cmp_internal(arg1, arg2));
 }
 
+Datum
+btfloat48cmp(PG_FUNCTION_ARGS)
+{
+	float4		arg1 = PG_GETARG_FLOAT4(0);
+	float8		arg2 = PG_GETARG_FLOAT8(1);
+
+	/* widen float4 to float8 and then compare */
+	PG_RETURN_INT32(float8_cmp_internal(arg1, arg2));
+}
+
+Datum
+btfloat84cmp(PG_FUNCTION_ARGS)
+{
+	float8		arg1 = PG_GETARG_FLOAT8(0);
+	float4		arg2 = PG_GETARG_FLOAT4(1);
+
+	/* widen float4 to float8 and then compare */
+	PG_RETURN_INT32(float8_cmp_internal(arg1, arg2));
+}
+
 
 /*
  *		===================
@@ -802,7 +1106,9 @@ dtoi4(PG_FUNCTION_ARGS)
 	int32		result;
 
 	if ((num < INT_MIN) || (num > INT_MAX))
-		elog(ERROR, "dtoi4: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int32) rint(num);
 	PG_RETURN_INT32(result);
@@ -819,7 +1125,9 @@ dtoi2(PG_FUNCTION_ARGS)
 	int16		result;
 
 	if ((num < SHRT_MIN) || (num > SHRT_MAX))
-		elog(ERROR, "dtoi2: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
 
 	result = (int16) rint(num);
 	PG_RETURN_INT16(result);
@@ -864,7 +1172,9 @@ ftoi4(PG_FUNCTION_ARGS)
 	int32		result;
 
 	if ((num < INT_MIN) || (num > INT_MAX))
-		elog(ERROR, "ftoi4: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	result = (int32) rint(num);
 	PG_RETURN_INT32(result);
@@ -881,7 +1191,9 @@ ftoi2(PG_FUNCTION_ARGS)
 	int16		result;
 
 	if ((num < SHRT_MIN) || (num > SHRT_MAX))
-		elog(ERROR, "ftoi2: integer out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
 
 	result = (int16) rint(num);
 	PG_RETURN_INT16(result);
@@ -1115,7 +1427,9 @@ dsqrt(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg1 < 0)
-		elog(ERROR, "can't take sqrt of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_POWER_FUNCTION),
+				 errmsg("cannot take square root of a negative number")));
 
 	result = sqrt(arg1);
 
@@ -1149,8 +1463,18 @@ dpow(PG_FUNCTION_ARGS)
 	float8		result;
 
 	/*
-	 * We must check both for errno getting set and for a NaN result, in
-	 * order to deal with the vagaries of different platforms...
+	 * The SQL spec requires that we emit a particular SQLSTATE error code for
+	 * certain error conditions.
+	 */
+	if ((arg1 == 0 && arg2 < 0) ||
+		(arg1 < 0 && floor(arg2) != arg2))
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_POWER_FUNCTION),
+				 errmsg("invalid argument for power function")));
+
+	/*
+	 * We must check both for errno getting set and for a NaN result, in order
+	 * to deal with the vagaries of different platforms...
 	 */
 	errno = 0;
 	result = pow(arg1, arg2);
@@ -1159,7 +1483,9 @@ dpow(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "pow() result is out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("result is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1176,9 +1502,9 @@ dexp(PG_FUNCTION_ARGS)
 	float8		result;
 
 	/*
-	 * We must check both for errno getting set and for a NaN result, in
-	 * order to deal with the vagaries of different platforms. Also, a
-	 * zero result implies unreported underflow.
+	 * We must check both for errno getting set and for a NaN result, in order
+	 * to deal with the vagaries of different platforms. Also, a zero result
+	 * implies unreported underflow.
 	 */
 	errno = 0;
 	result = exp(arg1);
@@ -1187,7 +1513,9 @@ dexp(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "exp() result is out of range");
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("result is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1196,7 +1524,6 @@ dexp(PG_FUNCTION_ARGS)
 
 /*
  *		dlog1			- returns the natural logarithm of arg1
- *						  ("dlog" is already a logging routine...)
  */
 Datum
 dlog1(PG_FUNCTION_ARGS)
@@ -1204,10 +1531,18 @@ dlog1(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		result;
 
+	/*
+	 * Emit particular SQLSTATE error codes for ln(). This is required by the
+	 * SQL standard.
+	 */
 	if (arg1 == 0.0)
-		elog(ERROR, "can't take log of zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_LOG),
+				 errmsg("cannot take logarithm of zero")));
 	if (arg1 < 0)
-		elog(ERROR, "can't take log of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_LOG),
+				 errmsg("cannot take logarithm of a negative number")));
 
 	result = log(arg1);
 
@@ -1225,10 +1560,19 @@ dlog10(PG_FUNCTION_ARGS)
 	float8		arg1 = PG_GETARG_FLOAT8(0);
 	float8		result;
 
+	/*
+	 * Emit particular SQLSTATE error codes for log(). The SQL spec doesn't
+	 * define log(), but it does define ln(), so it makes sense to emit the
+	 * same error code for an analogous error condition.
+	 */
 	if (arg1 == 0.0)
-		elog(ERROR, "can't take log of zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_LOG),
+				 errmsg("cannot take logarithm of zero")));
 	if (arg1 < 0)
-		elog(ERROR, "can't take log of a negative number");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_ARGUMENT_FOR_LOG),
+				 errmsg("cannot take logarithm of a negative number")));
 
 	result = log10(arg1);
 
@@ -1253,7 +1597,9 @@ dacos(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "acos(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1276,7 +1622,9 @@ dasin(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "asin(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1299,7 +1647,9 @@ datan(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "atan(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1323,7 +1673,9 @@ datan2(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "atan2(%f,%f) input is out of range", arg1, arg2);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1346,7 +1698,9 @@ dcos(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "cos(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1369,7 +1723,9 @@ dcot(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "cot(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	result = 1.0 / result;
 	CheckFloat8Val(result);
@@ -1393,7 +1749,9 @@ dsin(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "sin(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1416,7 +1774,9 @@ dtan(PG_FUNCTION_ARGS)
 		|| !finite(result)
 #endif
 		)
-		elog(ERROR, "tan(%f) input is out of range", arg1);
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("input is out of range")));
 
 	CheckFloat8Val(result);
 	PG_RETURN_FLOAT8(result);
@@ -1540,8 +1900,6 @@ float8_accum(PG_FUNCTION_ARGS)
 	float8		N,
 				sumX,
 				sumX2;
-	Datum		transdatums[3];
-	ArrayType  *result;
 
 	transvalues = check_float8_array(transarray, "float8_accum");
 	N = transvalues[0];
@@ -1552,15 +1910,34 @@ float8_accum(PG_FUNCTION_ARGS)
 	sumX += newval;
 	sumX2 += newval * newval;
 
-	transdatums[0] = Float8GetDatumFast(N);
-	transdatums[1] = Float8GetDatumFast(sumX);
-	transdatums[2] = Float8GetDatumFast(sumX2);
+	/*
+	 * If we're invoked by nodeAgg, we can cheat and modify our first
+	 * parameter in-place to reduce palloc overhead. Otherwise we construct a
+	 * new array with the updated transition data and return it.
+	 */
+	if (fcinfo->context && IsA(fcinfo->context, AggState))
+	{
+		transvalues[0] = N;
+		transvalues[1] = sumX;
+		transvalues[2] = sumX2;
 
-	result = construct_array(transdatums, 3,
-							 FLOAT8OID,
-						 sizeof(float8), false /* float8 byval */ , 'd');
+		PG_RETURN_ARRAYTYPE_P(transarray);
+	}
+	else
+	{
+		Datum		transdatums[3];
+		ArrayType  *result;
 
-	PG_RETURN_ARRAYTYPE_P(result);
+		transdatums[0] = Float8GetDatumFast(N);
+		transdatums[1] = Float8GetDatumFast(sumX);
+		transdatums[2] = Float8GetDatumFast(sumX2);
+
+		result = construct_array(transdatums, 3,
+								 FLOAT8OID,
+							 sizeof(float8), false /* float8 byval */ , 'd');
+
+		PG_RETURN_ARRAYTYPE_P(result);
+	}
 }
 
 Datum
@@ -1573,8 +1950,6 @@ float4_accum(PG_FUNCTION_ARGS)
 				sumX,
 				sumX2,
 				newval;
-	Datum		transdatums[3];
-	ArrayType  *result;
 
 	transvalues = check_float8_array(transarray, "float4_accum");
 	N = transvalues[0];
@@ -1588,15 +1963,34 @@ float4_accum(PG_FUNCTION_ARGS)
 	sumX += newval;
 	sumX2 += newval * newval;
 
-	transdatums[0] = Float8GetDatumFast(N);
-	transdatums[1] = Float8GetDatumFast(sumX);
-	transdatums[2] = Float8GetDatumFast(sumX2);
+	/*
+	 * If we're invoked by nodeAgg, we can cheat and modify our first
+	 * parameter in-place to reduce palloc overhead. Otherwise we construct a
+	 * new array with the updated transition data and return it.
+	 */
+	if (fcinfo->context && IsA(fcinfo->context, AggState))
+	{
+		transvalues[0] = N;
+		transvalues[1] = sumX;
+		transvalues[2] = sumX2;
 
-	result = construct_array(transdatums, 3,
-							 FLOAT8OID,
-						 sizeof(float8), false /* float8 byval */ , 'd');
+		PG_RETURN_ARRAYTYPE_P(transarray);
+	}
+	else
+	{
+		Datum		transdatums[3];
+		ArrayType  *result;
 
-	PG_RETURN_ARRAYTYPE_P(result);
+		transdatums[0] = Float8GetDatumFast(N);
+		transdatums[1] = Float8GetDatumFast(sumX);
+		transdatums[2] = Float8GetDatumFast(sumX2);
+
+		result = construct_array(transdatums, 3,
+								 FLOAT8OID,
+							 sizeof(float8), false /* float8 byval */ , 'd');
+
+		PG_RETURN_ARRAYTYPE_P(result);
+	}
 }
 
 Datum
@@ -1634,12 +2028,9 @@ float8_variance(PG_FUNCTION_ARGS)
 	sumX = transvalues[1];
 	sumX2 = transvalues[2];
 
-	/* We define VARIANCE of no values to be NULL, of 1 value to be 0 */
-	if (N == 0.0)
-		PG_RETURN_NULL();
-
+	/* Sample variance is undefined when N is 0 or 1, so return NULL */
 	if (N <= 1.0)
-		PG_RETURN_FLOAT8(0.0);
+		PG_RETURN_NULL();
 
 	numerator = N * sumX2 - sumX * sumX;
 
@@ -1665,12 +2056,9 @@ float8_stddev(PG_FUNCTION_ARGS)
 	sumX = transvalues[1];
 	sumX2 = transvalues[2];
 
-	/* We define STDDEV of no values to be NULL, of 1 value to be 0 */
-	if (N == 0.0)
-		PG_RETURN_NULL();
-
+	/* Sample stddev is undefined when N is 0 or 1, so return NULL */
 	if (N <= 1.0)
-		PG_RETURN_FLOAT8(0.0);
+		PG_RETURN_NULL();
 
 	numerator = N * sumX2 - sumX * sumX;
 
@@ -1738,7 +2126,9 @@ float48div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float48div: divide by zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 	CheckFloat8Val(result);
@@ -1798,7 +2188,9 @@ float84div(PG_FUNCTION_ARGS)
 	float8		result;
 
 	if (arg2 == 0.0)
-		elog(ERROR, "float84div: divide by zero");
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
 
 	result = arg1 / arg2;
 
@@ -1928,111 +2320,7 @@ float84ge(PG_FUNCTION_ARGS)
 
 /* ========== PRIVATE ROUTINES ========== */
 
-/* From "fdlibm" @ netlib.att.com */
-
-#ifndef HAVE_RINT
-
-/* @(#)s_rint.c 5.1 93/09/24 */
-/*
- * ====================================================
- * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
- *
- * Developed at SunPro, a Sun Microsystems, Inc. business.
- * Permission to use, copy, modify, and distribute this
- * software is freely granted, provided that this notice
- * is preserved.
- * ====================================================
- */
-
-/*
- * rint(x)
- * Return x rounded to integral value according to the prevailing
- * rounding mode.
- * Method:
- *		Using floating addition.
- * Exception:
- *		Inexact flag raised if x not equal to rint(x).
- */
-
-static const double one = 1.0,
-			TWO52[2] = {
-	4.50359962737049600000e+15, /* 0x43300000, 0x00000000 */
-	-4.50359962737049600000e+15,	/* 0xC3300000, 0x00000000 */
-};
-
-static double
-rint(double x)
-{
-	int			i0,
-				n0,
-				j0,
-				sx;
-	unsigned	i,
-				i1;
-	double		w,
-				t;
-
-	n0 = (*((int *) &one) >> 29) ^ 1;
-	i0 = *(n0 + (int *) &x);
-	sx = (i0 >> 31) & 1;
-	i1 = *(1 - n0 + (int *) &x);
-	j0 = ((i0 >> 20) & 0x7ff) - 0x3ff;
-	if (j0 < 20)
-	{
-		if (j0 < 0)
-		{
-			if (((i0 & 0x7fffffff) | i1) == 0)
-				return x;
-			i1 |= (i0 & 0x0fffff);
-			i0 &= 0xfffe0000;
-			i0 |= ((i1 | -i1) >> 12) & 0x80000;
-			*(n0 + (int *) &x) = i0;
-			w = TWO52[sx] + x;
-			t = w - TWO52[sx];
-			i0 = *(n0 + (int *) &t);
-			*(n0 + (int *) &t) = (i0 & 0x7fffffff) | (sx << 31);
-			return t;
-		}
-		else
-		{
-			i = (0x000fffff) >> j0;
-			if (((i0 & i) | i1) == 0)
-				return x;		/* x is integral */
-			i >>= 1;
-			if (((i0 & i) | i1) != 0)
-			{
-				if (j0 == 19)
-					i1 = 0x40000000;
-				else
-					i0 = (i0 & (~i)) | ((0x20000) >> j0);
-			}
-		}
-	}
-	else if (j0 > 51)
-	{
-		if (j0 == 0x400)
-			return x + x;		/* inf or NaN */
-		else
-			return x;			/* x is integral */
-	}
-	else
-	{
-		i = ((unsigned) (0xffffffff)) >> (j0 - 20);
-		if ((i1 & i) == 0)
-			return x;			/* x is integral */
-		i >>= 1;
-		if ((i1 & i) != 0)
-			i1 = (i1 & (~i)) | ((0x40000000) >> (j0 - 20));
-	}
-	*(n0 + (int *) &x) = i0;
-	*(1 - n0 + (int *) &x) = i1;
-	w = TWO52[sx] + x;
-	return w - TWO52[sx];
-}
-#endif   /* !HAVE_RINT */
-
 #ifndef HAVE_CBRT
-
 static double
 cbrt(double x)
 {
