@@ -1,7 +1,7 @@
 /*
  * conversion functions between pg_wchar and multibyte streams.
  * Tatsuo Ishii
- * $Id: wchar.c,v 1.34.2.1 2005/12/24 10:40:55 ishii Exp $
+ * $PostgreSQL: pgsql/src/backend/utils/mb/wchar.c,v 1.47 2005/10/29 00:31:52 petere Exp $
  *
  * WIN1250 client encoding updated by Pavel Behal
  *
@@ -49,9 +49,16 @@ pg_ascii_mblen(const unsigned char *s)
 	return (1);
 }
 
+static int
+pg_ascii_dsplen(const unsigned char *s)
+{
+	return (1);
+}
+
 /*
  * EUC
  */
+
 static int	pg_euc2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
 {
@@ -59,26 +66,26 @@ static int	pg_euc2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 2)	/* JIS X 0201 (so called "1 byte KANA") */
+		if (*from == SS2 && len >= 2)
 		{
 			from++;
-			*to = (SS2 << 8) | *from++;
+			*to = 0xff & *from++;
 			len -= 2;
 		}
-		else if (*from == SS3 && len >= 3)		/* JIS X 0212 KANJI */
+		else if (*from == SS3 && len >= 3)
 		{
 			from++;
-			*to = (SS3 << 16) | (*from++ << 8);
-			*to |= *from++;
+			*to = *from++ << 8;
+			*to |= 0x3f & *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* JIS X 0208 KANJI */
+		else if ((*from & 0x80) && len >= 2)
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
 			len -= 2;
 		}
-		else	/* must be ASCII */
+		else
 		{
 			*to = *from++;
 			len--;
@@ -106,6 +113,22 @@ pg_euc_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_euc_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s == SS2)
+		len = 2;
+	else if (*s == SS3)
+		len = 2;
+	else if (*s & 0x80)
+		len = 2;
+	else
+		len = 1;
+	return (len);
+}
+
 /*
  * EUC_JP
  */
@@ -121,6 +144,22 @@ pg_eucjp_mblen(const unsigned char *s)
 	return (pg_euc_mblen(s));
 }
 
+static int
+pg_eucjp_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s == SS2)
+		len = 1;
+	else if (*s == SS3)
+		len = 2;
+	else if (*s & 0x80)
+		len = 2;
+	else
+		len = 1;
+	return (len);
+}
+
 /*
  * EUC_KR
  */
@@ -136,9 +175,14 @@ pg_euckr_mblen(const unsigned char *s)
 	return (pg_euc_mblen(s));
 }
 
+static int
+pg_euckr_dsplen(const unsigned char *s)
+{
+	return (pg_euc_dsplen(s));
+}
+
 /*
  * EUC_CN
- *
  */
 static int	pg_euccn2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
@@ -147,21 +191,21 @@ static int	pg_euccn2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 3)	/* code set 2 (unused?) */
+		if (*from == SS2 && len >= 3)
 		{
 			from++;
-			*to = (SS2 << 16) | (*from++ << 8);
-			*to |= *from++;
+			*to = 0x3f00 & (*from++ << 8);
+			*to = *from++;
 			len -= 3;
 		}
-		else if (*from == SS3 && len >= 3)		/* code set 3 (unsed ?) */
+		else if (*from == SS3 && len >= 3)
 		{
 			from++;
-			*to = (SS3 << 16) | (*from++ << 8);
-			*to |= *from++;
+			*to = *from++ << 8;
+			*to |= 0x3f & *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* code set 1 */
+		else if ((*from & 0x80) && len >= 2)
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -191,9 +235,20 @@ pg_euccn_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_euccn_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s & 0x80)
+		len = 2;
+	else
+		len = 1;
+	return (len);
+}
+
 /*
  * EUC_TW
- *
  */
 static int	pg_euctw2wchar_with_len
 			(const unsigned char *from, pg_wchar *to, int len)
@@ -202,22 +257,22 @@ static int	pg_euctw2wchar_with_len
 
 	while (len > 0 && *from)
 	{
-		if (*from == SS2 && len >= 4)	/* code set 2 */
+		if (*from == SS2 && len >= 4)
 		{
 			from++;
-			*to = (SS2 << 24) | (*from++ << 16) ;
+			*to = *from++ << 16;
 			*to |= *from++ << 8;
 			*to |= *from++;
 			len -= 4;
 		}
-		else if (*from == SS3 && len >= 3)		/* code set 3 (unused?) */
+		else if (*from == SS3 && len >= 3)
 		{
 			from++;
-			*to = (SS3 << 16) | (*from++ << 8);
-			*to |= *from++;
+			*to = *from++ << 8;
+			*to |= 0x3f & *from++;
 			len -= 3;
 		}
-		else if ((*from & 0x80) && len >= 2)	/* code set 2 */
+		else if ((*from & 0x80) && len >= 2)
 		{
 			*to = *from++ << 8;
 			*to |= *from++;
@@ -251,6 +306,22 @@ pg_euctw_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_euctw_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s == SS2)
+		len = 2;
+	else if (*s == SS3)
+		len = 2;
+	else if (*s & 0x80)
+		len = 2;
+	else
+		len = 1;
+	return (len);
+}
+
 /*
  * JOHAB
  */
@@ -266,8 +337,14 @@ pg_johab_mblen(const unsigned char *s)
 	return (pg_euc_mblen(s));
 }
 
+static int
+pg_johab_dsplen(const unsigned char *s)
+{
+	return (pg_euc_dsplen(s));
+}
+
 /*
- * convert UTF-8 string to pg_wchar (UCS-2)
+ * convert UTF8 string to pg_wchar (UCS-2)
  * caller should allocate enough space for "to"
  * len: length of from.
  * "from" not necessarily null terminated.
@@ -318,7 +395,7 @@ pg_utf2wchar_with_len(const unsigned char *from, pg_wchar *to, int len)
 }
 
 /*
- * returns the byte length of a UTF-8 word pointed to by s
+ * returns the byte length of a UTF8 word pointed to by s
  */
 int
 pg_utf_mblen(const unsigned char *s)
@@ -329,11 +406,23 @@ pg_utf_mblen(const unsigned char *s)
 		len = 1;
 	else if ((*s & 0xe0) == 0xc0)
 		len = 2;
-	else if ((*s & 0xe0) == 0xe0)
+	else if ((*s & 0xf0) == 0xe0)
 		len = 3;
+	else if ((*s & 0xf8) == 0xf0)
+		len = 4;
+	else if ((*s & 0xfc) == 0xf8)
+		len = 5;
+	else if ((*s & 0xfe) == 0xfc)
+		len = 6;
 	return (len);
 }
 
+static int
+pg_utf_dsplen(const unsigned char *s)
+{
+	return 1;					/* XXX fix me! */
+}
+
 /*
  * convert mule internal code to pg_wchar
  * caller should allocate enough space for "to"
@@ -407,6 +496,12 @@ pg_mule_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_mule_dsplen(const unsigned char *s)
+{
+	return 1;					/* XXX fix me! */
+}
+
 /*
  * ISO8859-1
  */
@@ -431,6 +526,12 @@ pg_latin1_mblen(const unsigned char *s)
 	return (1);
 }
 
+static int
+pg_latin1_dsplen(const unsigned char *s)
+{
+	return (1);
+}
+
 /*
  * SJIS
  */
@@ -454,6 +555,26 @@ pg_sjis_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_sjis_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s >= 0xa1 && *s <= 0xdf)
+	{							/* 1 byte kana? */
+		len = 1;
+	}
+	else if (*s > 0x7f)
+	{							/* kanji? */
+		len = 2;
+	}
+	else
+	{							/* should be ASCII */
+		len = 1;
+	}
+	return (len);
+}
+
 /*
  * Big5
  */
@@ -473,6 +594,22 @@ pg_big5_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_big5_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s > 0x7f)
+	{							/* kanji? */
+		len = 2;
+	}
+	else
+	{							/* should be ASCII */
+		len = 1;
+	}
+	return (len);
+}
+
 /*
  * GBK
  */
@@ -492,6 +629,22 @@ pg_gbk_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_gbk_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s > 0x7f)
+	{							/* kanji? */
+		len = 2;
+	}
+	else
+	{							/* should be ASCII */
+		len = 1;
+	}
+	return (len);
+}
+
 /*
  * UHC
  */
@@ -511,6 +664,22 @@ pg_uhc_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_uhc_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s > 0x7f)
+	{							/* 2byte? */
+		len = 2;
+	}
+	else
+	{							/* should be ASCII */
+		len = 1;
+	}
+	return (len);
+}
+
 /*
  *	* GB18030
  *	 * Added by Bill Huang <bhuang@redhat.com>,<bill_huanghb@ybb.ne.jp>
@@ -536,42 +705,57 @@ pg_gb18030_mblen(const unsigned char *s)
 	return (len);
 }
 
+static int
+pg_gb18030_dsplen(const unsigned char *s)
+{
+	int			len;
+
+	if (*s <= 0x7f)
+	{							/* ASCII */
+		len = 1;
+	}
+	else
+		len = 2;
+	return (len);
+}
+
 
 pg_wchar_tbl pg_wchar_table[] = {
-	{pg_ascii2wchar_with_len, pg_ascii_mblen, 1},		/* 0; PG_SQL_ASCII	*/
-	{pg_eucjp2wchar_with_len, pg_eucjp_mblen, 3},		/* 1; PG_EUC_JP */
-	{pg_euccn2wchar_with_len, pg_euccn_mblen, 3},		/* 2; PG_EUC_CN */
-	{pg_euckr2wchar_with_len, pg_euckr_mblen, 3},		/* 3; PG_EUC_KR */
-	{pg_euctw2wchar_with_len, pg_euctw_mblen, 3},		/* 4; PG_EUC_TW */
-	{pg_johab2wchar_with_len, pg_johab_mblen, 3},		/* 5; PG_JOHAB */
-	{pg_utf2wchar_with_len, pg_utf_mblen, 3},	/* 6; PG_UNICODE */
-	{pg_mule2wchar_with_len, pg_mule_mblen, 3}, /* 7; PG_MULE_INTERNAL */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 8; PG_LATIN1 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 9; PG_LATIN2 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 10; PG_LATIN3 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 11; PG_LATIN4 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 12; PG_LATIN5 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 13; PG_LATIN6 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 14; PG_LATIN7 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 15; PG_LATIN8 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 16; PG_LATIN9 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 17; PG_LATIN10 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 18; PG_WIN1256 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 19; PG_TCVN */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 20; PG_WIN874 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 21; PG_KOI8 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 22; PG_WIN1251 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 23; PG_ALT */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 24; ISO-8859-5 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 25; ISO-8859-6 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 26; ISO-8859-7 */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 27; ISO-8859-8 */
-	{0, pg_sjis_mblen, 2},		/* 28; PG_SJIS */
-	{0, pg_big5_mblen, 2},		/* 29; PG_BIG5 */
-	{0, pg_gbk_mblen, 2},		/* 30; PG_GBK */
-	{0, pg_uhc_mblen, 2},		/* 31; PG_UHC */
-	{pg_latin12wchar_with_len, pg_latin1_mblen, 1},		/* 32; PG_WIN1250 */
-	{0, pg_gb18030_mblen, 2}	/* 33; PG_GB18030 */
+	{pg_ascii2wchar_with_len, pg_ascii_mblen, pg_ascii_dsplen, 1},		/* 0; PG_SQL_ASCII	*/
+	{pg_eucjp2wchar_with_len, pg_eucjp_mblen, pg_eucjp_dsplen, 3},		/* 1; PG_EUC_JP */
+	{pg_euccn2wchar_with_len, pg_euccn_mblen, pg_euccn_dsplen, 3},		/* 2; PG_EUC_CN */
+	{pg_euckr2wchar_with_len, pg_euckr_mblen, pg_euckr_dsplen, 3},		/* 3; PG_EUC_KR */
+	{pg_euctw2wchar_with_len, pg_euctw_mblen, pg_euctw_dsplen, 3},		/* 4; PG_EUC_TW */
+	{pg_johab2wchar_with_len, pg_johab_mblen, pg_johab_dsplen, 3},		/* 5; PG_JOHAB */
+	{pg_utf2wchar_with_len, pg_utf_mblen, pg_utf_dsplen, 4},	/* 6; PG_UTF8 */
+	{pg_mule2wchar_with_len, pg_mule_mblen, pg_mule_dsplen, 3}, /* 7; PG_MULE_INTERNAL */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 8; PG_LATIN1 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 9; PG_LATIN2 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 10; PG_LATIN3 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 11; PG_LATIN4 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 12; PG_LATIN5 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 13; PG_LATIN6 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 14; PG_LATIN7 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 15; PG_LATIN8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 16; PG_LATIN9 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 17; PG_LATIN10 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 18; PG_WIN1256 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 19; PG_WIN1258 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 20; PG_WIN874 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 21; PG_KOI8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 22; PG_WIN1251 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 22; PG_WIN1252 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 23; PG_WIN866 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 24; ISO-8859-5 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 25; ISO-8859-6 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 26; ISO-8859-7 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 27; ISO-8859-8 */
+	{pg_latin12wchar_with_len, pg_latin1_mblen, pg_latin1_dsplen, 1},	/* 28; PG_WIN1250 */
+	{0, pg_sjis_mblen, pg_sjis_dsplen, 2},		/* 29; PG_SJIS */
+	{0, pg_big5_mblen, pg_big5_dsplen, 2},		/* 30; PG_BIG5 */
+	{0, pg_gbk_mblen, pg_gbk_dsplen, 2},		/* 31; PG_GBK */
+	{0, pg_uhc_mblen, pg_uhc_dsplen, 2},		/* 32; PG_UHC */
+	{0, pg_gb18030_mblen, pg_gb18030_dsplen, 2} /* 33; PG_GB18030 */
 };
 
 /* returns the byte length of a word for mule internal code */
@@ -585,14 +769,28 @@ pg_mic_mblen(const unsigned char *mbstr)
  * Returns the byte length of a multibyte word.
  */
 int
-pg_encoding_mblen(int encoding, const unsigned char *mbstr)
+pg_encoding_mblen(int encoding, const char *mbstr)
 {
 	Assert(PG_VALID_ENCODING(encoding));
 
 	return ((encoding >= 0 &&
 			 encoding < sizeof(pg_wchar_table) / sizeof(pg_wchar_tbl)) ?
-			((*pg_wchar_table[encoding].mblen) (mbstr)) :
-			((*pg_wchar_table[PG_SQL_ASCII].mblen) (mbstr)));
+		((*pg_wchar_table[encoding].mblen) ((const unsigned char *) mbstr)) :
+	((*pg_wchar_table[PG_SQL_ASCII].mblen) ((const unsigned char *) mbstr)));
+}
+
+/*
+ * Returns the display length of a multibyte word.
+ */
+int
+pg_encoding_dsplen(int encoding, const char *mbstr)
+{
+	Assert(PG_VALID_ENCODING(encoding));
+
+	return ((encoding >= 0 &&
+			 encoding < sizeof(pg_wchar_table) / sizeof(pg_wchar_tbl)) ?
+	   ((*pg_wchar_table[encoding].dsplen) ((const unsigned char *) mbstr)) :
+	((*pg_wchar_table[PG_SQL_ASCII].dsplen) ((const unsigned char *) mbstr)));
 }
 
 /*
@@ -608,6 +806,60 @@ pg_encoding_max_length(int encoding)
 
 #ifndef FRONTEND
 
+bool
+pg_utf8_islegal(const unsigned char *source, int length)
+{
+	unsigned char a;
+	const unsigned char *srcptr = source + length;
+
+	switch (length)
+	{
+		default:
+			return false;
+			/* Everything else falls through when "true"... */
+		case 4:
+			if ((a = (*--srcptr)) < 0x80 || a > 0xBF)
+				return false;
+		case 3:
+			if ((a = (*--srcptr)) < 0x80 || a > 0xBF)
+				return false;
+		case 2:
+			if ((a = (*--srcptr)) > 0xBF)
+				return false;
+			switch (*source)
+			{
+					/* no fall-through in this inner switch */
+				case 0xE0:
+					if (a < 0xA0)
+						return false;
+					break;
+				case 0xED:
+					if (a > 0x9F)
+						return false;
+					break;
+				case 0xF0:
+					if (a < 0x90)
+						return false;
+					break;
+				case 0xF4:
+					if (a > 0x8F)
+						return false;
+					break;
+				default:
+					if (a < 0x80)
+						return false;
+			}
+
+		case 1:
+			if (*source >= 0x80 && *source < 0xC2)
+				return false;
+	}
+	if (*source > 0xF4)
+		return false;
+	return true;
+}
+
+
 /*
  * Verify mbstr to make sure that it has a valid character sequence.
  * mbstr is not necessarily NULL terminated; length of mbstr is
@@ -617,7 +869,7 @@ pg_encoding_max_length(int encoding)
  * true; when noError is false, ereport() a descriptive message.
  */
 bool
-pg_verifymbstr(const unsigned char *mbstr, int len, bool noError)
+pg_verifymbstr(const char *mbstr, int len, bool noError)
 {
 	int			l;
 	int			i;
@@ -631,51 +883,55 @@ pg_verifymbstr(const unsigned char *mbstr, int len, bool noError)
 
 	while (len > 0 && *mbstr)
 	{
-		/* special UTF-8 check */
-		if (encoding == PG_UTF8 && (*mbstr & 0xf8) == 0xf0)
-		{
-			if (noError)
-				return false;
-			ereport(ERROR,
-					(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
-			 errmsg("Unicode characters greater than or equal to 0x10000 are not supported")));
-		}
-
 		l = pg_mblen(mbstr);
 
-		for (i = 1; i < l; i++)
+		/* special UTF-8 check */
+		if (encoding == PG_UTF8)
 		{
-			/*
-			 * we expect that every multibyte char consists of bytes
-			 * having the 8th bit set
-			 */
-			if (i >= len || (mbstr[i] & 0x80) == 0)
+			if (!pg_utf8_islegal((const unsigned char *) mbstr, l))
 			{
-				char		buf[8 * 2 + 1];
-				char	   *p = buf;
-				int			j,
-							jlimit;
-
 				if (noError)
 					return false;
-
-				jlimit = Min(l, len);
-				jlimit = Min(jlimit, 8);		/* prevent buffer overrun */
-
-				for (j = 0; j < jlimit; j++)
-					p += sprintf(p, "%02x", mbstr[j]);
-
 				ereport(ERROR,
 						(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
-						 errmsg("invalid byte sequence for encoding \"%s\": 0x%s",
-								GetDatabaseEncodingName(), buf)));
+						 errmsg("invalid UTF-8 byte sequence detected near byte 0x%02x",
+								(unsigned char) *mbstr)));
 			}
 		}
+		else
+		{
+			for (i = 1; i < l; i++)
+			{
+				/*
+				 * we expect that every multibyte char consists of bytes
+				 * having the 8th bit set
+				 */
+				if (i >= len || (mbstr[i] & 0x80) == 0)
+				{
+					char		buf[8 * 2 + 1];
+					char	   *p = buf;
+					int			j,
+								jlimit;
 
+					if (noError)
+						return false;
+
+					jlimit = Min(l, len);
+					jlimit = Min(jlimit, 8);	/* prevent buffer overrun */
+
+					for (j = 0; j < jlimit; j++)
+						p += sprintf(p, "%02x", (unsigned char) mbstr[j]);
+
+					ereport(ERROR,
+							(errcode(ERRCODE_CHARACTER_NOT_IN_REPERTOIRE),
+					errmsg("invalid byte sequence for encoding \"%s\": 0x%s",
+						   GetDatabaseEncodingName(), buf)));
+				}
+			}
+		}
 		len -= l;
 		mbstr += l;
 	}
-
 	return true;
 }
 
@@ -689,6 +945,3 @@ pg_database_encoding_max_length(void)
 }
 
 #endif
-
-
-
